//
//	NSString+MiscRegex.m 
//         Regular expression matching and replacement routines using regexpr.c
//
//		Written by Carl Lindberg Copyright (c) 1996 by Carl Lindberg.
//			         Version 0.9  All rights reserved.
//		    This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "NSString+MiscRegex.h"
#import <misckit/regexpr.h>
#import <foundation/NSException.h>
#import <foundation/NSValue.h>
#import <foundation/NSCharacterSet.h>
#import <foundation/NSArray.h>
#import <foundation/NSDictionary.h>
#import <foundation/NSUtilities.h>
#import <stdlib.h>
#import <string.h>
#import <misckit/MiscStopwatch.h>

NSString *MiscRegexException = @"MiscRegexException";

NSString *MiscBeforeKey      = @"before";
NSString *MiscMiddleKey      = @"middle";
NSString *MiscAfterKey       = @"after";
NSString *MiscBeforeRangeKey = @"beforeRange";
NSString *MiscMiddleRangeKey = @"middleRange";
NSString *MiscAfterRangeKey  = @"afterRange";
NSString *MiscSubstringsKey  = @"substrings";
NSString *MiscSubrangesKey   = @"subranges";

static NSRange _rangeFromReg(regexp_registers_t regs, int idx);
static void _initRegex(NSString *regexString, regexp_t pattern,
					char translate[], char fastMap[], int options);

@interface NSString (MiscRegex_Private)
- (NSArray *)_substringsForRegisters:(regexp_registers_t)registers;
- (NSArray *) _subrangesForRegisters:(regexp_registers_t)registers;
- (NSString *)_processedStringFor:(NSString *)replaceString
                          options:(unsigned)mask
                        registers:(regexp_registers_t)registers;
- (BOOL)_findRegex:(NSString*)regex options:(unsigned)mask occurrenceNum:(int)n
         registers:(regexp_registers_t)registers;
@end

@implementation NSString (MiscRegex)

- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask occurrenceNum:(int)n
{
	NSRange foundRange = { NSNotFound, 0 };
	struct re_registers	registers;
	
	if ([self _findRegex:regex options:mask occurrenceNum:n registers:&registers])
	{
		foundRange = _rangeFromReg(&registers,0);
	}

	return foundRange;
}

- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask occurrenceNum:(int)n 
                  range:(NSRange)range
{
	NSString *subString = [self substringFromRange:range];
	NSRange	 theRange = [subString rangeOfRegex:regex options:mask occurrenceNum:n];
	
	if (theRange.location != NSNotFound) theRange.location += range.location;
	
	return theRange;
}

- grep:(NSString *)regex forKey:(NSString *)key options:(unsigned)mask occurrenceNum:(int)n
{
	NSArray		 *keyArray = [NSArray arrayWithObject:key];
	NSDictionary *theDict = [self grep:regex forKeys:keyArray options:mask occurrenceNum:n];
	return [theDict objectForKey:key];
}


- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys
			options:(unsigned)mask occurrenceNum:(int)n
{
	struct re_registers	registers;
	NSRange			foundRange = { NSNotFound, 0 };
	NSString		*currKey;
	NSEnumerator	*keyEnumerator = [keys objectEnumerator];
	NSMutableDictionary *resultDict = [NSMutableDictionary
										dictionaryWithCapacity:[keys count]];

	if ([self _findRegex:regex options:mask occurrenceNum:n registers:&registers])
		foundRange = _rangeFromReg(&registers,0);
	
	while (currKey = [keyEnumerator nextObject])
	{
		id theObject = nil;

		if ([currKey isEqualToString:MiscBeforeKey]) {
			if (foundRange.location == NSNotFound)
				theObject = self;
			else
				theObject = [self substringToIndex:foundRange.location];
		}
		
		else if ([currKey isEqualToString:MiscMiddleKey]) {
			if (foundRange.location == NSNotFound)
				theObject = [NSString stringWithCString:""];
			else
				theObject = [self substringFromRange:foundRange];
		}
		
		else if ([currKey isEqualToString:MiscAfterKey]) {
			if (foundRange.location == NSNotFound)
				theObject = [NSString stringWithCString:""];
			else
				theObject = [self substringFromIndex:NSMaxRange(foundRange)];
		}
		
		else if ([currKey isEqualToString:MiscBeforeRangeKey]) {
			NSRange aRange = {0, [self length]};

			if (foundRange.location != NSNotFound)
				aRange.length = foundRange.location;

			theObject = [NSValue value:&aRange withObjCType:@encode(NSRange)];
		}
		
		else if ([currKey isEqualToString:MiscMiddleRangeKey]) {
			theObject = [NSValue value:&foundRange withObjCType:@encode(NSRange)];
		}
		
		else if ([currKey isEqualToString:MiscAfterRangeKey]) {
			NSRange aRange = foundRange;
			
			if (foundRange.location != NSNotFound) {
				aRange.location = NSMaxRange(foundRange);
				aRange.length = [self cStringLength] - aRange.location;
			}
			
			theObject = [NSValue value:&aRange withObjCType:@encode(NSRange)];
		}
		
		else if ([currKey isEqualToString:MiscSubstringsKey]) {
			if (foundRange.location == NSNotFound)
				theObject = [NSArray array];
			else
				theObject = [self _substringsForRegisters:&registers];
		}
		
		else if ([currKey isEqualToString:MiscSubrangesKey]) {
			if (foundRange.location == NSNotFound)
				theObject = [NSArray array];
			else
				theObject = [self _subrangesForRegisters:&registers];
		}
		
		if (theObject)
			[resultDict setObject:theObject forKey:currKey];
	}
	
	return resultDict;
}

- (NSArray*)rangesMatchedByRegex:(NSString *)regex options:(unsigned)mask
{
	struct re_pattern_buffer	pattern;
	struct re_registers			registers;
	char		translate[256], fastMap[256];
	char		*myString = (char*)[self cString];
	int			myLength = [self cStringLength];
	int			currSpot = 0;
	int			lastEnd = 0;
	int			posFound = -1;
	NSRange		foundRange;
	NSMutableArray	*rangeArray = [NSMutableArray array];
	NSValue		*currValue;
	
	_initRegex(regex, &pattern, translate, fastMap, mask);

	while ((currSpot <= myLength) &&
	       (posFound = re_search_pattern(&pattern, myString, myLength,
		                         currSpot, myLength-currSpot, &registers)) >= 0)
	{
		foundRange = _rangeFromReg(&registers, 0);

        /*
         * We don't want to have a zero-length match at the same spot a
         * previous match ended.  So, start looking at the next spot.
         */
		if (lastEnd == foundRange.location && foundRange.length == 0 && currSpot > 0) {
			currSpot++;
			continue;
		}

		currValue = [NSValue value:&foundRange withObjCType:@encode(NSRange)];
		[rangeArray addObject:currValue];
		
		lastEnd = currSpot = NSMaxRange(foundRange);
		if(foundRange.length == 0) currSpot++;
	}
	
	if (posFound == -2)
		[NSException raise:MiscRegexException format:@"Error during regex search"];

	if (pattern.buffer) free(pattern.buffer);	
	return rangeArray;
}

- (NSArray*)stringsMatchedByRegex:(NSString *)regex options:(unsigned)mask
{
	NSRange			stringRange;
	NSEnumerator	*rangeEnum;
	NSArray			*rangesArray;
	NSMutableArray	*stringsArray = [NSMutableArray array];
	NSValue			*currValue;
	
	rangesArray = [self rangesMatchedByRegex:regex options:mask];
	rangeEnum = [rangesArray objectEnumerator];

	while (currValue = [rangeEnum nextObject])
	{
		[currValue getValue:&stringRange];
		[stringsArray addObject:[self substringFromRange:stringRange]];
	}
	
	return stringsArray;
}

- (NSArray *)componentsSeparatedByRegex:(NSString *)regex options:(unsigned)mask
{
	NSRange			matchRange, stringRange;
	NSEnumerator	*rangeEnum;
	NSArray			*rangesArray;
	NSMutableArray	*stringsArray = [NSMutableArray array];
	NSValue			*currValue;
	unsigned		lastMatchEnd = 0;
	
	rangesArray = [self rangesMatchedByRegex:regex options:mask];
	rangeEnum = [rangesArray objectEnumerator];

	while (currValue = [rangeEnum nextObject])
	{
		[currValue getValue:&matchRange];

		stringRange.location = lastMatchEnd;
		stringRange.length = matchRange.location - stringRange.location;
		
		[stringsArray addObject:[self substringFromRange:stringRange]];
		
		lastMatchEnd = NSMaxRange(matchRange);
	}
	
	stringRange.location = lastMatchEnd;
	stringRange.length = [self cStringLength] - lastMatchEnd;
	[stringsArray addObject:[self substringFromRange:stringRange]];

	return stringsArray;
}

- (id)stringByReplacingRegex:(NSString *)regex withString:(NSString *)replaceString
                     options:(unsigned)mask occurrenceNum:(int)n
{
	struct re_registers registers;
	NSRange		theRange;
	NSString	*realReplaceString;
	
	if ([self _findRegex:regex options:mask occurrenceNum:n registers:&registers])
	{
		realReplaceString = [self _processedStringFor:replaceString
										options:mask registers:&registers];

		theRange = _rangeFromReg(&registers, 0);

		return [NSString stringWithFormat:@"%@%@%@",
					[self substringToIndex:theRange.location],
					realReplaceString,
					[self substringFromIndex:NSMaxRange(theRange)]];
	}

	return self;
}


- (id)stringByReplacingRegex:(NSString *)regex withString:(NSString *)replaceString 
		options:(unsigned)mask occurrenceNum:(int)n range:(NSRange)range
{
	NSString *subString = [self substringFromRange:range];
	NSString *replacement = [subString stringByReplacingRegex:regex withString:replaceString
								options:mask occurrenceNum:n];
	
	return [NSString stringWithFormat:@"%@%@%@",
				[self substringToIndex:range.location],
				replacement,
				[self substringFromIndex:NSMaxRange(range)]];
}


- (NSString *)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex
              withString:(NSString *)aString options:(unsigned)mask
{
	struct re_pattern_buffer	pattern;
	struct re_registers			registers;
	char		translate[256], fastMap[256];
	char		*myString = (char*)[self cString];
	int			myLength = [self cStringLength];
	int			numFound = 0;
	int			currSpot = 0;
	int			lastEnd = 0;
	int			posFound = -1;
	NSRange		foundRange, betweenRange;
	NSMutableString	*newString = [NSMutableString stringWithCapacity:[self length]];
	NSString	*replaceString;
	
	_initRegex(regex, &pattern, translate, fastMap, mask);
	
	while ((currSpot <= myLength) && (posFound = re_search_pattern(&pattern, myString,
						myLength, currSpot, myLength-currSpot, &registers)) >= 0)
	{
		foundRange            = _rangeFromReg(&registers, 0);
		betweenRange.location = lastEnd;
		betweenRange.length   = foundRange.location - lastEnd;
		
		if (betweenRange.length == 0 && foundRange.length == 0 && currSpot > 0) {
			currSpot++;
			continue;
		}
		
		if (betweenRange.length > 0)
			[newString appendString:[self substringFromRange:betweenRange]];
		
		replaceString = [self _processedStringFor:aString options:mask registers:&registers];			
		[newString appendString:replaceString];

		lastEnd = currSpot = NSMaxRange(foundRange);
		if(foundRange.length == 0) currSpot++;
		numFound++;
	}
	
	if (posFound == -2)
		[NSException raise:MiscRegexException format:@"Error during regex search"];

	if (lastEnd < myLength)
		[newString appendString:[self substringFromIndex:lastEnd]];

	if (pattern.buffer) free(pattern.buffer);
	
	return newString;
}

- (NSString *)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex 
              withString:(NSString *)aString options:(unsigned)mask range:(NSRange)range
{
	NSString *subString = [self substringFromRange:range];
	NSString *replacement = [subString stringByReplacingEveryOccurrenceOfRegex:regex
								withString:aString options:mask];
	
	return [NSString stringWithFormat:@"%@%@%@",
				[self substringToIndex:range.location],
				replacement,
				[self substringFromIndex:NSMaxRange(range)]];
}

- (unsigned)numOfRegex:(NSString *)regex options:(unsigned)mask
{	// These should do raw searches; this is inefficient
	NSArray	*stringArray = [self stringsMatchedByRegex:regex options:mask];
	return [stringArray count];
}

- (unsigned)numOfRegex:(NSString *)regex options:(unsigned)mask range:(NSRange)range
{
	return [[self substringFromRange:range] numOfRegex:regex options:mask];
}

-(BOOL)isValidRegex;
/*
 * Unfortunately, re_compile_pattern() doesn't catch all possible problems
 * -- it accepts things like "a**", which will cause a runtime error when
 * searching.  Most errors are caught though.  Still, I might change the
 * implementation to be an NS_DURING/NSHANDLER around a call to _findRegex
 * or something, so the above errors would be caught.  I wanted to avoid
 * having to do that, but...
 */
{
	struct re_pattern_buffer pattern;
	char	translate[256];
	char	*errorString;
	int		i;

	memset(&pattern, 0, sizeof(struct re_pattern_buffer));
	for (i=0;i<256;i++) translate[i] = i;
	pattern.translate = translate;
	
	errorString = re_compile_pattern((char*)[self cString], [self cStringLength], &pattern);
	if (pattern.buffer) free(pattern.buffer);
	
	if (errorString)
		return NO;
	else
		return YES;
}



/*******************************************************************/
/**************   Convenience methods ******************************/
/*******************************************************************/

- (unsigned)numOfRegex:(NSString *)regex
{ return [self numOfRegex:regex options:0];
}
- (unsigned)numOfRegex:(NSString *)regex range:(NSRange)range
{ return [self numOfRegex:regex options:0 range:range];
}


- (NSRange)rangeOfRegex:(NSString *)regex
{ return [self rangeOfRegex:regex options:0 occurrenceNum:0];
}
- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask
{ return [self rangeOfRegex:regex options:mask occurrenceNum:0];
}
- (NSRange)rangeOfRegex:(NSString *)regex occurrenceNum:(int)n
{ return [self rangeOfRegex:regex options:0 occurrenceNum:n];
}
- (NSRange)rangeOfRegex:(NSString *)regex range:(NSRange)range
{ return [self rangeOfRegex:regex options:0 occurrenceNum:0 range:range];
}
- (NSRange)rangeOfRegex:(NSString *)regex options:(unsigned)mask range:(NSRange)range
{ return [self rangeOfRegex:regex options:mask occurrenceNum:0 range:range];
}
- (NSRange)rangeOfRegex:(NSString *)regex occurrenceNum:(int)n range:(NSRange)range
{ return [self rangeOfRegex:regex options:0 occurrenceNum:n range:range];
}


- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys
{ return [self grep:regex forKeys:keys options:0 occurrenceNum:0];
}
- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys options:(unsigned)mask
{ return [self grep:regex forKeys:keys options:mask occurrenceNum:0];
}
- (NSDictionary *)grep:(NSString *)regex forKeys:(NSArray *)keys occurrenceNum:(int)n
{ return [self grep:regex forKeys:keys options:0 occurrenceNum:n];
}

- grep:(NSString *)regex forKey:(NSString *)keys
{ return [self grep:regex forKey:keys options:0 occurrenceNum:0];
}
- grep:(NSString *)regex forKey:(NSString *)keys options:(unsigned)mask
{ return [self grep:regex forKey:keys options:mask occurrenceNum:0];
}
- grep:(NSString *)regex forKey:(NSString *)keys occurrenceNum:(int)n
{ return [self grep:regex forKey:keys options:0 occurrenceNum:n];
}


- (id)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex withString:(NSString *)aString
{ return [self stringByReplacingEveryOccurrenceOfRegex:regex withString:aString options:0];
}
- (id)stringByReplacingEveryOccurrenceOfRegex:(NSString *)regex withString:(NSString *)aString 
				range:(NSRange)range
{ return [self stringByReplacingEveryOccurrenceOfRegex:regex withString:aString options:0 
				range:range];
}


- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString
{ return [self stringByReplacingRegex:regex withString:replaceString options:0  
				occurrenceNum:0];
}
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString  
				options:(unsigned)mask
{ return [self stringByReplacingRegex:regex withString:replaceString options:mask  
				occurrenceNum:0];
}
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString  
				occurrenceNum:(int)n
{ return [self stringByReplacingRegex:regex withString:replaceString options:0  
				occurrenceNum:n];
}
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString 
				range:(NSRange)range
{ return [self stringByReplacingRegex:regex withString:replaceString options:0 occurrenceNum:0 
				range:range];
}
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString  
				options:(unsigned)mask range:(NSRange)range
{ return [self stringByReplacingRegex:regex withString:replaceString options:mask  
				occurrenceNum:0 range:range];
}
- (id)stringByReplacingRegex:(NSString*)regex withString:(NSString*)replaceString  
				occurrenceNum:(int)n range:(NSRange)range
{ return [self stringByReplacingRegex:regex withString:replaceString options:0 occurrenceNum:n  
				range:range];
}


- (NSArray*)stringsMatchedByRegex:(NSString *)regex
{ return [self stringsMatchedByRegex:regex options:0];
}
- (NSArray*)rangesMatchedByRegex:(NSString *)regex
{ return [self rangesMatchedByRegex:regex options:0];
}
- (NSArray*)componentsSeparatedByRegex:(NSString *)regex
{ return [self componentsSeparatedByRegex:regex options:0];
}


@end



#define NS_CHARSET(str) [NSCharacterSet characterSetWithCharactersInString:str]

static NSRange _rangeFromReg(regexp_registers_t regs, int idx)
{
	NSRange theRange = {NSNotFound, 0};
	
	if (regs->start[idx] >= 0)
	{
		theRange.location = regs->start[idx];
		theRange.length   = regs->end[idx] - theRange.location;
	}

	return theRange;
}


static void _initRegex(NSString *regexString, regexp_t pattern,
					char translate[], char fastMap[], int options)
{
	int		i;
	char	*errorString;

	if (!regexString) {
		[NSException raise:MiscRegexException format:@"_initRegex(): nil regex string"];
	}

	memset(pattern, 0, sizeof(struct re_pattern_buffer));
	
	for (i=0;i<256;i++) translate[i] = i;
	if (options & NSCaseInsensitiveSearch)
	{
		for (i='A'; i<='Z'; i++)	translate[i] = i - 'A' + 'a';
	}
	pattern->translate = translate;
	
	if (errorString = re_compile_pattern((char*)[regexString cString],
						[regexString cStringLength], pattern))
	{	
		[NSException raise:MiscRegexException format:
			@"_initRegex(): Malformed regular expression '%@': %s", regexString, errorString];
	}

	if ((options & MiscFasterSearch) && fastMap) {
//		memset(fastMap,0,256);  // the regexpr code does this
		pattern->fastmap = fastMap;
		re_compile_fastmap(pattern);
	}
}


@implementation NSString (MiscRegex_Private)

- (NSString *)_processedStringFor:(NSString *)replaceString
                          options:(unsigned)mask
                        registers:(regexp_registers_t)registers
{
	int				len               = [replaceString length];
	NSMutableString	*processedString  = [NSMutableString stringWithCapacity:len];
	NSCharacterSet	*digitCharacters  = [NSCharacterSet decimalDigitCharacterSet];
	NSCharacterSet	*quotingChars     = NS_CHARSET(@"$");
	NSCharacterSet	*ampersandSet     = NS_CHARSET(@"&");
	NSCharacterSet	*needToQuoteChars = NS_CHARSET(@"&$");
	unichar			aChar, nextChar;
	NSRange			theRange;
	int				matchNum;
	unsigned		i;

	if (!(mask & MiscUseMatchSubstitutions)) return replaceString;

	// This probably could be written better
	for (i=0; i<len; i++)
	{
		matchNum = -1;
		aChar = [replaceString characterAtIndex:i];

		if ([ampersandSet characterIsMember:aChar]) {
			matchNum = 0;
		}
		else if ([quotingChars characterIsMember:aChar] && (i+1 < len))
		{
			nextChar = [replaceString characterAtIndex:i+1];

			if ([digitCharacters characterIsMember:nextChar]) {
				matchNum = nextChar - '0';
				i++;
			}
			else if ([needToQuoteChars characterIsMember:nextChar]) {
				aChar = nextChar;
				i++;
			}
		}
		
		
		if (matchNum >= 0) {
			if ((matchNum < RE_NREGS) && (registers->start[matchNum] >= 0)) {
				theRange = _rangeFromReg(registers, matchNum);
				[processedString appendString:[self substringFromRange:theRange]];
			}
		}
		else {
			[processedString appendFormat:@"%c",aChar];
		}
	}
		
	return processedString;
}

	
- (BOOL)_findRegex:(NSString*)regex options:(unsigned)mask occurrenceNum:(int)n
	     registers:(regexp_registers_t)registers
{
	struct re_pattern_buffer	pattern;
	char		translate[256], fastMap[256];
	char		*myString = (char*)[self cString];
	int			myLength = [self cStringLength];
	int			numFound = 0;
	int			currSpot = 0;
	int			lastEnd = 0;
	int			posFound = -1, matchLen = -1;

	if (mask & NSBackwardsSearch) {
		int num = [self numOfRegex:regex options:mask];
		n = num - 1 - n;
	}
	
	if (n < 0) {
		return NO;
	}

	_initRegex(regex, &pattern, translate, fastMap, mask);
	
	while ((numFound <= n) && (currSpot <= myLength) &&
	       ((posFound = re_search_pattern(&pattern, myString, myLength, currSpot,
		                 myLength-currSpot, registers)) >= 0))
	{
		matchLen = _rangeFromReg(registers, 0).length;
	
		if (matchLen == 0 && currSpot == lastEnd && currSpot > 0) {
			currSpot++;
			continue;
		}

		if (matchLen >= 0 && numFound <= n) {
			lastEnd=currSpot = posFound + matchLen;
			if (matchLen == 0) currSpot++;
			numFound++;
		}
		else {
			posFound = -1;
			break;
		}
	}
	if (pattern.buffer) free(pattern.buffer);

	if (n >= numFound) posFound = -1;

	if (posFound == -2 || matchLen == -2)
		[NSException raise:MiscRegexException format:@"Error while searching"];

	if (posFound < 0) return NO;

	if (mask & NSAnchoredSearch)
	{
		if (mask & NSBackwardsSearch) {
			if (registers->end[0] == [self cStringLength]) return YES;
		}
		else {
			if (registers->start[0] == 0) return YES;
		}
		
		return NO;
	}

	return YES;
}

- (NSArray *)_substringsForRegisters:(regexp_registers_t)registers
{
	NSMutableArray	*substringArray = [NSMutableArray array];
	NSRange			currRange;
	int				highestFound, i;
	
	for (i=0,highestFound=0; i < RE_NREGS; i++)
		if (registers->start[i] >= 0) highestFound = i;

	for (i=0; i <= highestFound; i++)
	{
		if (registers->start[i] >= 0) {
			currRange = _rangeFromReg(registers,i);
			[substringArray addObject:[self substringFromRange:currRange]];
		}
		else {
			[substringArray addObject:[NSString stringWithCString:""]];
		}
	}
	
	return substringArray;
}

- (NSArray *)_subrangesForRegisters:(regexp_registers_t)registers
{
	NSMutableArray	*rangesArray = [NSMutableArray array];
	NSRange			currRange;
	NSValue			*currValue;
	int				highestFound, i;
	
	for (i=0,highestFound=0; i < RE_NREGS; i++)
		if (registers->start[i] >= 0) highestFound = i;

	for (i=0; i <= highestFound; i++)
	{
		currRange = _rangeFromReg(registers, i);
		currValue = [NSValue value:&currRange withObjCType:@encode(NSRange)];
		[rangesArray addObject:currValue];
	}
	
	return rangesArray;
}

@end

